@@ -85,8 +85,30 @@
 #define h3 h.h32[3]
 #define h4 hh
 
+/* For fat builds */
+#if HAVE_NATIVE_poly1305_set_key
 void
-_poly1305_set_key(struct poly1305_ctx *ctx, const uint8_t key[16])
+_nettle_poly1305_set_key_c(struct poly1305_ctx *ctx,
+	       const uint8_t key[16]);
+# define _nettle_poly1305_set_key _nettle_poly1305_set_key_c
+#endif
+
+#if HAVE_NATIVE_poly1305_block
+void
+_nettle_poly1305_block_c(struct poly1305_ctx *ctx, const uint8_t *m,
+	       unsigned t4);
+# define _nettle_poly1305_block _nettle_poly1305_block_c
+#endif
+
+#if HAVE_NATIVE_poly1305_digest
+void
+_nettle_poly1305_digest_c(struct poly1305_ctx *ctx,
+	       union nettle_block16 *s);
+# define _nettle_poly1305_digest _nettle_poly1305_digest_c
+#endif
+
+void
+_nettle_poly1305_set_key(struct poly1305_ctx *ctx, const uint8_t key[16])
 {
   uint32_t t0,t1,t2,t3;
 
@@ -114,7 +136,7 @@ _poly1305_set_key(struct poly1305_ctx *ctx, const uint8_t key[16])
 }
 
 void
-_poly1305_block (struct poly1305_ctx *ctx, const uint8_t *m, unsigned t4)
+_nettle_poly1305_block (struct poly1305_ctx *ctx, const uint8_t *m, unsigned t4)
 {
   uint32_t t0,t1,t2,t3;
   uint32_t b;
@@ -149,7 +171,7 @@ _poly1305_block (struct poly1305_ctx *ctx, const uint8_t *m, unsigned t4)
 
 /* Adds digest to the nonce */
 void
-_poly1305_digest (struct poly1305_ctx *ctx, union nettle_block16 *s)
+_nettle_poly1305_digest (struct poly1305_ctx *ctx, union nettle_block16 *s)
 {
   uint32_t b, nb;
   uint64_t f0,f1,f2,f3;
