@@ -1,6 +1,6 @@
-/* sha3-224-meta.c
+/* sha512-224-meta.c
 
-   Copyright (C) 2012 Niels Möller
+   Copyright (C) 2014 Niels Möller
 
    This file is part of GNU Nettle.
 
@@ -35,8 +35,15 @@
 
 #include "nettle-meta.h"
 
-#include "sha3.h"
-#include "sha3-internal.h"
+#include "sha2.h"
+
+const struct nettle_hash nettle_sha512_224 =
+  {
+    "sha512_224", sizeof(struct sha512_ctx),
+    SHA512_224_DIGEST_SIZE,
+    SHA512_224_BLOCK_SIZE,
+    (nettle_hash_init_func *) sha512_224_init,
+    (nettle_hash_update_func *) sha512_224_update,
+    (nettle_hash_digest_func *) sha512_224_digest
+  };
 
-const struct nettle_hash nettle_sha3_224
-= _NETTLE_HASH(sha3_224, SHA3_224);
